/***************************************************************************
                          intro.c
    description:          introduction
                             -------------------
    begin                : Fri March 30 2001
    date                 : 21/07/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <SDL/SDL.h>
#include "structs.h" // global screen
#include "data.h"
#include "sprite.h"
#include "intro.h"

static SDL_Surface *backgroundIntro;

/**
 * intro_init()
 * init intro vars.
 *
 * Returns:     -1 on error, otherwise a positive integer.
 *
**/
int intro_init(void)
{
	backgroundIntro = LoadImage(DATAFILE("backgroundIntro.gif"), 0);
	if (backgroundIntro)
		return 1;
	return -1;
}


/**
 * intro_free()
 * free intro vars.
 *
**/
void intro_free(void)
{
	SDL_FreeSurface(backgroundIntro);
}


//This is the main start screen
int intro_run(void)
{
	SDL_Event event;
	SDL_Rect drect;
	Uint8 *key;
	int leave_state = RUN_INTRO;

	//Paint the background
	drect.x = 0;
	drect.y = 0;
	SDL_BlitSurface(backgroundIntro, NULL, screen, &drect);
	SDL_UpdateRect(screen, 0, 0, 0, 0);

	while (leave_state == RUN_INTRO)
	{
		//Poll input queue, run keyboard loop
		while (SDL_PollEvent(&event))
		{
			if ( event.type == SDL_QUIT )
				leave_state = RUN_QUIT;
		}
		key = SDL_GetKeyState(NULL);
		if (key[SDLK_s] == SDL_PRESSED)
			leave_state = RUN_GAME;
		else if (key[SDLK_v] == SDL_PRESSED)
			leave_state = RUN_SCORE;
		else if (key[SDLK_q] == SDL_PRESSED || key[SDLK_ESCAPE] == SDL_PRESSED)
			leave_state = RUN_QUIT;
		WaitFrame();
	}
	return leave_state;
}
