/***************************************************************************
                          main.c
  description: the main loop
                             -------------------
    begin                : Fri March 30 2001
    date                 : 21/07/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>

#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_image.h>

#include "config.h"
#include "sound.h"
#include "structs.h"
#include "data.h"
#include "hi_score.h"
#include "final_score.h"
#include "sprite.h"
#include "rungame.h"
#include "intro.h"
#include "rcfile.h"

SDL_Surface *screen;
SDL_Surface *background;

int fullscreen;
char *rcfile;

static void _usage(void)
{
	printf("Usage:\n  %s [Option...]\n\n", PACKAGE_NAME);
	printf("  -?      \tshow this message\n");
	printf("  -v      \tprint version number\n");
	printf("  -f      \tfullscreen\n");
	printf("  -m    m \tbuiltin MOD music\n");
	printf("        w \tbuiltin WAV music\n");
	printf("        n \tno music\n");
	printf("  -t   <file>\tplay music file\n\n");
}

static int _get_rc_prefs(void)
{
	FILE *f = NULL;
	char *arg;

	rcfile = dsi_rc_init("dsirc", 1);
	if (rcfile == NULL)
		return 0;
	if ((f = dsi_open_prefs(rcfile, "[dsi-prefs]")) == NULL)
		return 0;

	if ((arg = dsi_get_tag_arg(f, "fullscreen")) != NULL)
	{
		int i = atoi(arg);
		if (i == 1 || i == 0)
			fullscreen = i;
	}
	if ((arg = dsi_get_tag_arg(f, "track-mode")) != NULL)
	{
		if (0 == strcmp(arg, "m"))
			music_decoder = MPLAY_MOD;
		else if (0 == strcmp(arg, "w"))
			music_decoder = MPLAY_WAV;
		else if (0 == strcmp(arg, "n"))
			music_decoder = MPLAY_NONE;
	}
	if ((arg = dsi_get_tag_arg(f, "track-file")) != NULL)
	{
		if (*arg) {
			music_file = malloc(strlen(arg)+1);
			if (music_file)
			{
				strcpy(music_file, arg);
				music_decoder = MPLAY_FILE;
			}
		}
	}
	fclose(f);
	return 1;
}

static int _init_all(void)
{
	int tally;

	if ((background = LoadImage(DATAFILE("background.gif"), 0)) == NULL)
		return -1;
	if ((tally = rungame_init()) < 0)
		return -1;
	if (sprite_init(tally) < 0)
		return -1;

	if (sound_init() < 0)
		return -1;
	if (intro_init() < 0)
		return -1;
	if (hi_score_init() < 0)
		return -1;
	if (final_score_init() < 0)
		return -1;
	return 1;
}

static void _free_all(void)
{
	rungame_free();
	sprite_free();
	intro_free();
	hi_score_free();
	final_score_free();
	sound_free();
	SDL_FreeSurface(background);
	free(music_file);
}

int main(int argc, char *argv[])
{
	int leave = 0, CarryScore = 0;
	int opt;
	music_decoder = MPLAY_WAV;
	music_file = NULL;
	SDL_Surface *image = NULL;
	SDL_Joystick *joy = NULL;

	//rcfile overrides defaults
	_get_rc_prefs();

	//Command line overrides rcfile
	while ((opt = getopt(argc, argv, "fv?m:t:")) != -1) {
		switch (opt) {
		case 'f':
			fullscreen = 1;
			break;
		case 'm':
			if (0 == strcmp(optarg, "m"))
				music_decoder = MPLAY_MOD;
			else if (0 == strcmp(optarg, "w"))
				music_decoder = MPLAY_WAV;
			else if (0 == strcmp(optarg, "n"))
				music_decoder = MPLAY_NONE;
			else
			{
			printf("%s invalid option to -m   '%s'\n",
					PACKAGE_NAME, optarg);
				_usage();
				return 0;
			}
			break;
		case 't':
			strncpy(music_file, optarg, PATH_MAX);
			music_file[PATH_MAX -1] = '\0';
			music_decoder = MPLAY_FILE;
			break;
		case 'v':
			printf("%s version is %s\n",
					PACKAGE_NAME, PACKAGE_VERSION);
			return 0;
		default: /* '?' */
			_usage();
			return 0;
		}
	}

	//Initialize the SDL library
	if ( SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO|SDL_INIT_JOYSTICK) < 0 )
	{
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(2);
	}

	//Open the audio device
	if ( Mix_OpenAudio(11025, AUDIO_U8, 1, 512) < 0 )
	{
		fprintf(stderr,
		"Warning: Couldn't set 11025 Hz 8-bit audio\n- Reason: %s\n",
							SDL_GetError());
	}
// seems to be a compile problem with GetNumMusicDecoders and getMusicDecoder
//	else
//	{
//		//Detect music decoders
//		int i, n;
//		n = Mix_GetNumMusicDecoders();
//		for (i = 0; i < n; ++i)
//		{
//			if (0 == strcmp(Mix_GetMusicDecoder(i), "MIKMOD"));
//				music_decoder = MPLAY_MOD;
//		}
//	}
	//Open the display device (if fullscreen -f flag selected
	//at command line)
	//Open the joy stick
#ifdef __LINUX__
	if (SDL_NumJoysticks())
		joy=SDL_JoystickOpen(0);
#endif
	if(fullscreen == 1)
	{
		screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE|SDL_FULLSCREEN);
		if ( screen == NULL )
		{
			fprintf(stderr, "Couldn't set 640x480 video mode: %s\n",
								SDL_GetError());
			exit(2);
		}
	}else
	{
#ifdef __LINUX__
		image = IMG_Load(DSI_PIXMAP_DIR "/alien.png");
#else
		image = SDL_LoadBMP("/Alien.ICO");
#endif
		if (image)
			SDL_WM_SetIcon(image, NULL);
		screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE);
		if ( screen == NULL )
		{
			fprintf(stderr, "Couldn't set 640x480 video mode: %s\n",
								SDL_GetError());
			exit(2);
		}
		SDL_WM_SetCaption("DSI Space Invaders", "DSI");
	}

	//Initialize the random number generator
	srand(time(NULL));

	//Load the artwork
	if (_init_all() > 0)
	{
		while (leave != RUN_QUIT)
		{
			if (leave == RUN_INTRO)
			{
				leave = intro_run();
			}
			if (leave == RUN_GAME)
			{
				CarryScore = 0;
				leave = RunGame(&CarryScore);
			}
			if (leave == RUN_FINAL)
			{
				leave = final_score_view(CarryScore);
			}
			//if 'v' for view score is pressed in intro and after game
			if (leave == RUN_SCORE)
			{
				leave = hi_score_view();
			}
		}
		//Free the artwork
		_free_all();
	}
	if(joy)
		SDL_JoystickClose(joy);
	Mix_CloseAudio();
	SDL_Quit();
	return 0;
}
