/***************************************************************************
                          sprite.c
    description:          sprite manipulations
                           -------------------
    begin                : Fri March 30 2001
    date                 : 10/07/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "structs.h" // global screen and background
#include "sprite.h"

SDL_Rect *dstupdate;
int numupdates;


//Prototypes
static void sprite_draw(Sprite *sprite);
static void sprite_erase(Sprite *sprite);
static void sprite_kill(Sprite *sprite);


int  sprite_init(int number)
{
	numupdates = 0;
	dstupdate = malloc(3 * number * sizeof(SDL_Rect));
	if (!dstupdate)
	{
		printf("Memory error.\n");
		return 0;
	}
	sprite_ops[SPRITE_DRAW] = sprite_draw;
	sprite_ops[SPRITE_ERASE] = sprite_erase;
	sprite_ops[SPRITE_KILL] = sprite_kill;
	return 1;
}


void sprite_free(void)
{
	free(dstupdate);
}

static void sprite_draw(Sprite *sprite)
{
	SDL_Rect  srect;
	SDL_Rect *drect;

	drect = &dstupdate[numupdates++];
	srect.x = 0;
	srect.y = 0;
	srect.w = drect->w = sprite->image->w;
	srect.h = drect->h = sprite->image->h;
	drect->x = sprite->x;
	drect->y = sprite->y;
	SDL_LowerBlit(sprite->image, &srect, screen, drect);
}


static void sprite_kill(Sprite *sprite)
{
	sprite->alive = 0;
}


static void sprite_erase(Sprite *sprite)
{
	SDL_Rect  srect;
	SDL_Rect *drect;

	drect = &dstupdate[numupdates++];
	srect.x = drect->x = sprite->x;
	srect.y = drect->y = sprite->y;
	srect.w = drect->w = sprite->image->w;
	srect.h = drect->h = sprite->image->h;
	SDL_LowerBlit(background, &srect, screen, drect);
}


int sprite_collide(Sprite *sprite1, Sprite *sprite2)
{
	if ( (sprite1->y >= (sprite2->y+sprite2->image->h)) ||
	     (sprite1->x >= (sprite2->x+sprite2->image->w)) ||
	     (sprite2->y >= (sprite1->y+sprite1->image->h)) ||
	     (sprite2->x >= (sprite1->x+sprite1->image->w)) )
	{
		return(0);
	}
	return(1);
}


void UpdateScreen(void)
{
	SDL_UpdateRects(screen, numupdates, dstupdate);
	numupdates = 0;
}


void WaitFrame(void)
{
	static Uint32 next_tick = 0;
	Uint32 this_tick;

	//Wait for the next frame
	this_tick = SDL_GetTicks();
	if ( this_tick < next_tick )
	{
		SDL_Delay(next_tick-this_tick);
	}
	next_tick = this_tick + (1000/FRAMES_PER_SEC);
}
